#include <stdio.h>
#include <stdlib.h>
#include "trimesh.h"

void read_tri_mesh(const char *filename, TriMesh *mesh)
{
   FILE *fp;
   int i;

   fp=fopen(filename, "rt");
   if(!fp) return;
   fscanf(fp, "%d", &mesh->nv);
   mesh->x=(float*)malloc(3*mesh->nv*sizeof(float));
   for(i=0; i<3*mesh->nv; ++i)
      fscanf(fp, "%g", mesh->x+i);
   fscanf(fp, "%d", &mesh->nt);
   mesh->tri=(int*)malloc(3*mesh->nt*sizeof(int));
   for(i=0; i<3*mesh->nt; ++i)
      fscanf(fp, "%d", mesh->tri+i);
   fclose(fp);
   mesh->deg=0;
   mesh->adj=0;
   mesh->bdry=0;
}

void write_tri_mesh(const char *filename, TriMesh *mesh)
{
   FILE *fp;
   int i;

   fp=fopen(filename, "wt");
   if(!fp) return;
   fprintf(fp, "%d\n", mesh->nv);
   for(i=0; i<mesh->nv; ++i)
      fprintf(fp, "%.7g %.7g %.7g\n", mesh->x[3*i], mesh->x[3*i+1], mesh->x[3*i+2]);
   fprintf(fp, "%d\n", mesh->nt);
   for(i=0; i<mesh->nt; ++i)
      fprintf(fp, "%d %d %d\n", mesh->tri[3*i], mesh->tri[3*i+1], mesh->tri[3*i+2]);
   fclose(fp);
}

static void insert_into_list(int *list, int *count, int entry)
{
   int i;
   for(i=0; i<*count; ++i)
      if(list[i]==entry) return;
   list[*count]=entry;
   ++*count;
}

void compute_adjacencies(TriMesh *mesh)
{
   int i, j, k, t;
   int *tricount;

   tricount=(int*)calloc(mesh->nv, sizeof(int));
   mesh->deg=(int*)calloc(mesh->nv, sizeof(int));
   mesh->adj=(int**)calloc(mesh->nv, sizeof(int*));
   mesh->bdry=(char*)calloc(mesh->nv, sizeof(char));

   // get upper bound on degrees
   for(t=0; t<mesh->nt; ++t){
      ++tricount[mesh->tri[3*t]];
      ++tricount[mesh->tri[3*t+1]];
      ++tricount[mesh->tri[3*t+2]];
   }
   // (over-)allocate space for the adjacencies
   for(i=0; i<mesh->nv; ++i)
      mesh->adj[i]=(int*)malloc(2*tricount[i]*sizeof(int));
   // get the adjacency lists, eliminating duplicates, and counting degrees
   for(t=0; t<mesh->nt; ++t){
      i=mesh->tri[3*t]; j=mesh->tri[3*t+1]; k=mesh->tri[3*t+2];
      insert_into_list(mesh->adj[i], mesh->deg+i, j);
      insert_into_list(mesh->adj[i], mesh->deg+i, k);
      insert_into_list(mesh->adj[j], mesh->deg+j, k);
      insert_into_list(mesh->adj[j], mesh->deg+j, i);
      insert_into_list(mesh->adj[k], mesh->deg+k, i);
      insert_into_list(mesh->adj[k], mesh->deg+k, j);
   }
   // see where the degree is more than the triangle count - those are boundary nodes
   // (side note: where the degree is strictly less - those are non-manifold nodes)
   for(i=0; i<mesh->nv; ++i){
      if(mesh->deg[i]>tricount[i])
         mesh->bdry[i]=1;
   }
   free(tricount);
}

