#ifndef CLOTH_H
#define CLOTH_H

#include "trimesh.h"

typedef struct{
   TriMesh mesh;
   float *m; // array of masses for each vertex
   float *v; // array of velocities for each vertex
   float *k; // array of spring constants for each edge
   float *L; // array of rest lengths of springs for each edge
   float *d; // array of spring damping constants for each edge
   float max_strain; // maximum strain allowed for any spring
   float *kbend; // array of constants for bending forces
   float *restbend; // array of rest position parameters for bending forces (sin(theta_0)/2)
   float *dbend; // array of constants for bending damping
} Cloth;

// assumes cloth->mesh is defined, rest isn't
void make_cloth(Cloth *cloth, float density, float planar_youngs, float planar_damping, float max_strain,
                float bending_resistance, float bending_damping);
float max_time_step(const Cloth *cloth);
void add_forces(const Cloth *cloth, const float *x, const float *v, float *f);
void add_elastic_spring_forces(const Cloth *cloth, const float *x, float *f);
void add_damping_spring_forces(const Cloth *cloth, const float *x, const float *v, float *f);
void add_elastic_bending_forces(const Cloth *cloth, const float *x, float *f);
void add_damping_bending_forces(const Cloth *cloth, const float *x, const float *v, float *f);
void limit_strain(const Cloth *cloth, float dt, float *x, float *v);

#endif
