// Example code for homework 4.

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cloth.h"
#include "levelset.h"

// global parameters for simulation
int end_seconds; // final time
const float frames_per_second=24;
float density, planar_youngs, planar_damping, max_strain, bending_resistance, bending_damping;
int collision_steps_per_frame;
float friction;
float gravity=-9.8;

// global variables
int frame=0;
float sim_t=0; // current time
Cloth cloth;
LevelSet object;

// functions
void read_parameters(const char *filename);
void init_simulation(const char *directory);
void advance_with_collisions(float end_time);

// start of definitions
int main(int argc, char **argv)
{
   int i, j;
   const char *directory=".";
   char filename[1000];

   if(argc<2 || argc>3){
      fprintf(stderr,"Need arguments: parameter-file [optional-directory]\n");
      return 1;
   }else if(argc==3)
      directory=argv[2];

   read_parameters(argv[1]);
   init_simulation(directory);

   while(frame/frames_per_second < end_seconds){
      ++frame;
      for(j=1; j<=collision_steps_per_frame; ++j){
         float end_t=(frame+j/(float)collision_steps_per_frame)/frames_per_second;
         advance_with_collisions(end_t);
         sim_t=end_t;
      }
      sprintf(filename, "%s/cloth%04d.mesh", directory, frame);
      write_tri_mesh(filename, &cloth.mesh);
      fputc(frame%10 ? '+' : '#', stdout); fflush(stdout);
   }
   return 0;
}

void read_parameters(const char *filename)
{
   FILE *fp=fopen(filename, "rt");
   fscanf(fp, "%d", &end_seconds);
   fscanf(fp, "%f", &density);
   fscanf(fp, "%f %f %f", &planar_youngs, &planar_damping, &max_strain);
   fscanf(fp, "%f %f", &bending_resistance, &bending_damping);
   fscanf(fp, "%d", &collision_steps_per_frame);
   fscanf(fp, "%f", &friction);
   fclose(fp);
}

void init_simulation(const char *directory)
{
   char filename[1000];

   sprintf(filename, "%s/object.levelset", directory);
   read_level_set(filename, &object);

   sprintf(filename, "%s/cloth0000.mesh", directory);
   read_tri_mesh(filename, &cloth.mesh);
   compute_auxiliary_structures(&cloth.mesh);
   make_cloth(&cloth, density, planar_youngs, planar_damping, max_strain, bending_resistance, bending_damping);
}

void advance_with_collisions(float end_t)
{
   // @@@ fill in code here that advances cloth with collisions, similar to homework 2.
   // The forces are gravity (9.8m/s^2 downwards in y) and the cloth internal forces (see cloth.h, add_forces())
   // After each symplectic Euler step, use limit_strain() from cloth.* to shrink over-stretched springs.
   // A certain number of times per frame, check for collisions just like homework 2.
   // You may want to add extra functions similar to those in homework 2.
}

