#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "trimesh.h"

void read_tri_mesh(const char *filename, TriMesh *mesh)
{
   FILE *fp;
   int i;

   fp=fopen(filename, "rt");
   if(!fp) return;
   fscanf(fp, "%d", &mesh->nv);
   mesh->x=(float*)malloc(3*mesh->nv*sizeof(float));
   for(i=0; i<3*mesh->nv; ++i)
      fscanf(fp, "%g", mesh->x+i);
   fscanf(fp, "%d", &mesh->nt);
   mesh->tri=(int*)malloc(3*mesh->nt*sizeof(int));
   for(i=0; i<3*mesh->nt; ++i)
      fscanf(fp, "%d", mesh->tri+i);
   fclose(fp);
   mesh->vnormals=0; // by default we don't compute these
}

void write_tri_mesh(const char *filename, TriMesh *mesh)
{
   FILE *fp;
   int i;

   fp=fopen(filename, "wt");
   if(!fp) return;
   fprintf(fp, "%d\n", mesh->nv);
   for(i=0; i<mesh->nv; ++i)
      fprintf(fp, "%.7g %.7g %.7g\n", mesh->x[3*i], mesh->x[3*i+1], mesh->x[3*i+2]);
   fprintf(fp, "%d\n", mesh->nt);
   for(i=0; i<mesh->nt; ++i)
      fprintf(fp, "%d %d %d\n", mesh->tri[3*i], mesh->tri[3*i+1], mesh->tri[3*i+2]);
   fclose(fp);
}

static inline void normalize_v(float v[3])
{
   float overmag=1.0/sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
   v[0]*=overmag;
   v[1]*=overmag;
   v[2]*=overmag;
}

static inline void add_v(float x[3], const float y[3])
{
   x[0]+=y[0];
   x[1]+=y[1];
   x[2]+=y[2];
}

void compute_vertex_normals(TriMesh *mesh)
{
   int i;

   mesh->vnormals=(float*)calloc(3*mesh->nv,sizeof(float));
   for(i=0; i<mesh->nt; ++i){
      int a=mesh->tri[3*i], b=mesh->tri[3*i+1], c=mesh->tri[3*i+2];
      float normal[3];
      normal[0]=(mesh->x[3*b+1]-mesh->x[3*a+1])*(mesh->x[3*c+2]-mesh->x[3*a+2])-(mesh->x[3*b+2]-mesh->x[3*a+2])*(mesh->x[3*c+1]-mesh->x[3*a+1]);
      normal[1]=(mesh->x[3*b+2]-mesh->x[3*a+2])*(mesh->x[3*c+0]-mesh->x[3*a+0])-(mesh->x[3*b+0]-mesh->x[3*a+0])*(mesh->x[3*c+2]-mesh->x[3*a+2]);
      normal[2]=(mesh->x[3*b+0]-mesh->x[3*a+0])*(mesh->x[3*c+1]-mesh->x[3*a+1])-(mesh->x[3*b+1]-mesh->x[3*a+1])*(mesh->x[3*c+0]-mesh->x[3*a+0]);
      normalize_v(normal);
      add_v(mesh->vnormals+3*a, normal);
      add_v(mesh->vnormals+3*b, normal);
      add_v(mesh->vnormals+3*c, normal);
   }
   for(i=0; i<mesh->nv; ++i)
      normalize_v(mesh->vnormals+3*i);
}

